# ==========================================================
# vikiwiki.rb -- VikiWiki-WEBrick Trial-Kit package.
#
# Copyright (C) 2005 Z80-OOLONG. (NAKATSUKA, Yukitaka)
# ==========================================================

############
# a fix and extension code of WEBrick module. ;-(

module WEBrick
  ############
  # class CGI extension for convertion from WEBrick::Request to CGI.

  class ::CGI
    alias __env_table__ env_table
    alias __initialize__ initialize

    ############
    # If type is "empty", CGI::new method creates a CGI object which has no
    # params and cookies.

    def initialize(type = "query")
      @alt_env_table = (type == "empty") ? {"REQUEST_METHOD" => "GET"} : nil
      __initialize__(type)
    end

    def env_table; @alt_env_table || __env_table__; end
    def set_env_table(env)
      if @alt_env_table then
        @alt_env_table.clear; @alt_env_table.update(env)
      end
    end
  end

  module HTTPAuth
    class BasicAuth
      def decode64(str); Base64::decode64(str); end
    end
  end

  class HTTPServer
    ############
    # Before calling method HTTPServer#mount, hook method servlet.mounted
    # is called if servlet class object respond to it.

    alias __mount__ mount
    def mount(dir, servlet, *options)
      if servlet.respond_to?(:mounted) then
        dir, *options = servlet.mounted(dir, *options)
      end
      __mount__(dir, servlet, *options)
    end
  end

  ############
  # class WEBrick::Output an object for output to WEBrick::Response.

  class Output
    def initialize(response)
      @response = response; @response.body = ""; @headbuf = ""
    end

    def <<(lines)
      lines.to_s.each_line("\n") do |line|
        if @headbuf && (/^\r*$/ =~ line) then
          begin
            headers = WEBrick::HTTPUtils::parse_header(@headbuf)
            if /^(\d+)/ =~ headers['status'][0] then
              @response.status = $1.to_i; headers.delete('status')
            end
            headers.each{|k,v| @response[k] = v.join(", ")}
          rescue => e
            raise WEBrick::HTTPStatus::InternalServerError, e.message
          end
          @headbuf = nil
        else
          (@headbuf || @response.body) << line
        end
      end
    end

    def print(*args)
      self << args.map {|arg| arg.to_s}.join
    end
  end

  class HTTPRequest
    def to_cgi
      cgi = ::CGI.new("empty")

      cgi.params.clear
      query.each do |k, v|
        cgi.params[k] = v.to_ary.map{|s| s.gsub!(/\r\n/, "\n"); s}
      end
      cgi.cookies.clear
      cgi.cookies.update(
        ::CGI::Cookie::parse(cookies.map{|c| c.to_s}.join("; "))
      )
      cgi.set_env_table(self.meta_vars)

      return cgi
    end
  end

  class HTTPResponse
    def to_out; Output::new(self); end
  end
end

# ==========================================================
